"use client";
import React, { useEffect, useState, useRef } from "react";

import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { z } from "zod";
import { Label } from "@/components/ui/label";
import { Input } from "@/components/ui/input";
import { Button } from "@/components/ui/button";
import { cn } from "@/lib/utils";
import { useTransition } from "react";
import { useTranslations } from "next-intl";
import toast, { Toaster } from "react-hot-toast";
import { postWithToken, getWithToken, postApiData } from "@/Helper/common";
import { useLanguage } from '../../../../../../context/LanguageContext'
import { useMediaQuery } from "@/hooks/use-media-query";
import { RadioGroup, RadioGroupItem } from "@/components/ui/radio-group";
import { useRouter } from "next/navigation";


const PasswordUpdate = () => {

  const { locale, changeLanguage, messages } = useLanguage();
  const router = useRouter();
  const totalOtpField = 4;
  const otpArray = Array.from({ length: totalOtpField }, () => "");
  const [otp, setOtp] = useState(otpArray);
  const otpFields = Array.from({ length: totalOtpField }, (_, index) => index);
  const inputRefs = useRef([]);

  const lang = useTranslations("UpdatePassword");
  const lang1 = useTranslations("UpdateMPin");
  const ProfileLang = useTranslations("Profile");

  const schema = z.object({
    oldPassword: z.string().min(3, lang("Enter_Old_Password")),
    newPassword: z.string().min(3, { message: lang("Enter_New_Password") }),
    confirmPassword: z
      .string()
      .min(3, { message: lang("Enter_Confirm_Password") }),
  });

  const {
    register,
    setValue,
    handleSubmit,
    reset,
    formState: { errors },
  } = useForm({
    resolver: zodResolver(schema),
  });

  var apiCall = true;
  const fetchKycData = async () => {
    try {

      if (apiCall) {

        apiCall = false;

        const apiResData = await getWithToken("get-kyc-details");
        if (apiResData.error === false) {

          setMobile(apiResData.userData?.phone);
          setEmail(apiResData.userData?.email);
          setUniqueId(apiResData.userData?.unique_id);
        }
      }
    } catch (error) {
      console.error("Error fetching:", error);

    }
  };

  useEffect(() => {
    fetchKycData();
  }, []);


  async function finalSubmit() {

    const data = formData;
    toast.dismiss();
    const apiData = JSON.stringify({
      old_password: data.oldPassword,
      new_password: data.newPassword,
      confirm_password: data.confirmPassword,
    });

    if (!data.oldPassword.trim()) {

      toast.error('Old password is required', { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });

    } else {

      try {

        setLoading(true);

        const data = await postWithToken("change-password", apiData);

        var messageshow = data.message;
        if (locale === "hi") {
          messageshow = messages.ErrorMessage[data.response_code];
        }

        if (data.error == false) {

          toast.success(messageshow, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });
          setLoading(false);
          setLoadingOtpVerify(false);
          setShowOTP(false);
          setValue("oldPassword", '');
          setValue("newPassword", '');
          setValue("confirmPassword", '');
          setOtp([]);

          router.push('/admin/dashboard');

        } else {

          setOtp([]);
          setLoading(false);
          toast.error(messageshow, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });

        }

      } catch (errorData) {

        setOtp([]);
        setLoading(false);
        toast.error(messageshow, { position: "bottom-center", style: { borderRadius: '10px', background: '#333', color: '#fff' } });
      }

    }
  }

  const [formData, setFormData] = useState([]);

  async function onSubmit(data) {
    setFormData(data);
    otpResend();
    setLoading(true);
  }

  const handleReset = () => {
    setValue("oldPassword", '');
    setValue("newPassword", '');
    setValue("confirmPassword", '');
    setOtp([]);
  }

  const [showOTP, setShowOTP] = useState(false);
  const [OTP, setOTP] = useState("");
  const [mobile, setMobile] = useState("");
  const [email, setEmail] = useState("");
  const [uniqueId, setUniqueId] = useState("");

  const [isPending, startTransition] = React.useState(false);
  const [passwordType, setPasswordType] = React.useState("password");
  const isDesktop2xl = useMediaQuery("(max-width: 1530px)");

  const [loadingOtpVerify, setLoadingOtpVerify] = useState(false);
  const [loadingOtpResend, setLoadingOtpResend] = useState(false);
  const [loading, setLoading] = useState(false);

  const [countdown, setCountdown] = useState(0);
  const [isCounting, setIsCounting] = useState(true);
  const [oldPassword, setOldPassword] = useState('');

  function formatMobileNumber(number) {
    if (otpType == 'mobile') {
      var formattedNumber = "xxxxxx" + number?.substring(6);
      return formattedNumber;
    } else {
      return "xxxxxx" + email?.substring(6);
    }
  }

  // otp verify
  async function onOTPVerify(event) {

    event.preventDefault();
    toast.dismiss();

    const OTP = otp.join("");

    const apiData = JSON.stringify({ phone: mobile, otp: OTP, unique_id: uniqueId, type: 'verify' });

    try {
      setLoadingOtpVerify(true);

      const data = await postApiData("admin/otp-verify", apiData);

      if (data.error == false) {
        // toast.success(data.message, {
        //   position: "bottom-center",
        //   style: { borderRadius: "10px", background: "#333", color: "#fff" },
        // });

        setOtp([]);
        setShowOTP(false);
        finalSubmit();
        setLoadingOtpVerify(false);
        setOtp(Array(otpFields.length).fill(''));
        router.back();

      } else {
        toast.error(data.message, {
          position: "bottom-center",
          style: { borderRadius: "10px", background: "#333", color: "#fff" },
        });
        setLoadingOtpVerify(false);
        setOtp(Array(otpFields.length).fill(''));
      }
    } catch (errorData) {
      toast.error(errorData.message, {
        position: "bottom-center",
        style: { borderRadius: "10px", background: "#333", color: "#fff" },
      });
      setLoadingOtpVerify(false);
      setOtp(Array(otpFields.length).fill(''));
    }

    // Add logic to verify OTP here
  }

  // send otp resend
  async function otpResend() {

    const dataPass = formData;
    setOtp([]);
    const apiData = JSON.stringify({ phone: mobile, unique_id: uniqueId, otpType, matchType: 'password', matchValue: oldPassword });
    toast.dismiss();
    setOtp(Array(otpFields.length).fill(''));
    try {
      setLoadingOtpResend(true);

      const data = await postApiData("admin/otp-resend", apiData);

      if (data.error == false) {
        toast.success(data.message, {
          position: "bottom-center",
          style: { borderRadius: "10px", background: "#333", color: "#fff" },
        });

        setShowOTP(true);
        setLoadingOtpResend(false);
        setLoading(false);
        setCountdown(120);
        setIsCounting(true);
      } else {
        toast.error(data.message, {
          position: "bottom-center",
          style: { borderRadius: "10px", background: "#333", color: "#fff" },
        });
        setLoadingOtpResend(false);
        setLoading(false);
      }
    } catch (errorData) {
      toast.error(errorData.message, {
        position: "bottom-center",
        style: { borderRadius: "10px", background: "#333", color: "#fff" },
      });
      setLoadingOtpResend(false);
      setLoading(false);
    }

    // Add logic to verify OTP here
  }



  const handleChange = (e, index) => {
    const { value } = e.target;
    if (!isNaN(value) && value.length <= 1) {
      const newOtp = [...otp];
      newOtp[index] = value;
      setOtp(newOtp);
      if (value.length === 1 && index < totalOtpField - 1) {
        inputRefs.current[index + 1].focus();
      }
    }
  };

  const handleKeyDown = (index, event) => {
    if (event.key === "Backspace" && otp[index] === "" && index > 0) {
      setOtp(prevOtp => {
        const newOtp = [...prevOtp];
        newOtp[index - 1] = "";
        return newOtp;
      });
      inputRefs.current[index - 1].focus();
    } else if (event.key === "ArrowLeft" && index > 0) {
      inputRefs.current[index - 1].focus();
    } else if (event.key === "ArrowRight" && index < numInputs - 1) {
      inputRefs.current[index + 1].focus();
    }
  };

  const isOtpComplete = otp.every((digit) => digit !== "");
  const [isHovered, setIsHovered] = useState(false);
  const [otpType, setOtpType] = useState('mobile');

  useEffect(() => {
    if (isCounting && countdown > 0) {
      var timer = setInterval(() => {
        setCountdown(prevCountdown => prevCountdown - 1);
      }, 1000);
    } else if (countdown === 0) {
      setIsCounting(false);
    }
    return () => clearInterval(timer);
  }, [isCounting, countdown]);

  const OtpConfirm = (value) => {

    // alert(value);
    setOtpType(value); // Set to true if confirmed

  };

  return (

    <>
      {!showOTP && (

        <form onSubmit={handleSubmit(onSubmit)}>
          <div className="grid grid-cols-1 lg:grid-cols-2 gap-4">
            <div className="flex flex-col gap-2">
              <Label htmlFor="Old Password">{lang("Enter_Old_Password")}</Label>
              <Input
                type="password"
                disabled={isPending}
                {...register("oldPassword", {
                  setValueAs: (value) => value.split(" ").join("")
                })}
                placeholder={lang("Enter_Old_Password")}
                onInput={(e) => {
                  e.target.value = e.target.value.split(" ").join("");
                  setOldPassword(e.target.value);
                }}
              />
              {errors.oldPassword && (
                <p
                  className={cn("text-xs", {
                    "text-destructive": errors.oldPassword,
                  })}
                >
                  {errors.oldPassword.message}
                </p>
              )}
            </div>

            <div className="flex flex-col gap-2">
              <Label htmlFor="New Password">{lang("Enter_New_Password")}</Label>
              <Input
                type="password"
                {...register("newPassword", {
                  setValueAs: (value) => value.split(" ").join("")
                })}
                placeholder={lang("Enter_New_Password")}
                disabled={isPending}
                onInput={(e) => {
                  e.target.value = e.target.value.split(" ").join("");
                }}
              />
              {errors.newPassword && (
                <p
                  className={cn("text-xs", {
                    "text-destructive": errors.newPassword,
                  })}
                >
                  {errors.newPassword.message}
                </p>
              )}
            </div>

            <div className="flex flex-col gap-2">
              <Label htmlFor="Confirm Password">
                {lang("Enter_Confirm_Password")}
              </Label>
              <Input
                type="password"
                {...register("confirmPassword", {
                  setValueAs: (value) => value.split(" ").join("")
                })}
                placeholder={lang("Enter_Confirm_Password")}
                disabled={isPending}
                onInput={(e) => {
                  e.target.value = e.target.value.split(" ").join("");
                }}
              />
              {errors.confirmPassword && (
                <p
                  className={cn("text-xs", {
                    "text-destructive": errors.confirmPassword,
                  })}
                >
                  {errors.confirmPassword.message}
                </p>
              )}
            </div>
            <div className="flex flex-col gap-2">
              <Label
                htmlFor="password"
                className="mb-2 font-medium text-default-600"
              >
                {ProfileLang("prefer_to_receive_your_verify_OTP")}:
              </Label>
              <div className="relative">
                <RadioGroup
                  className="mb-10 gap-5 md:gap-30 mt-1"
                  value={otpType == 'mobile' ? 'mobile' : 'email'} // Control value with state
                  onValueChange={OtpConfirm} // Handle value change
                >
                  <RadioGroupItem value="mobile" id="mobile">
                  {ProfileLang("Registered_Mobile")}
                  </RadioGroupItem>

                  <RadioGroupItem value="email" id="email">
                  {ProfileLang("Registered_Email")}
                  </RadioGroupItem>
                </RadioGroup>
              </div>
            </div>
          </div>

          <div className="mt-5 flex justify-center gap-10">
            <div>
              <Button onClick={handleReset} className=" mx-auto" type="button"> {lang("Reset")}</Button>
            </div>
            <div>
              <Button className=" mx-auto" type="submit" disabled={loading}> {lang("Submit")}
                {loading && (
                  <svg
                    xmlns="http://www.w3.org/2000/svg"
                    width="2em"
                    height="2em"
                    viewBox="0 0 24 24"
                  >
                    <g
                      fill="none"
                      stroke="currentColor"
                      strokeLinecap="round"
                      strokeWidth="2"
                    >
                      <path
                        strokeDasharray="60"
                        strokeDashoffset="60"
                        strokeOpacity="0.3"
                        d="M12 3C16.9706 3 21 7.02944 21 12C21 16.9706 16.9706 21 12 21C7.02944 21 3 16.9706 3 12C3 7.02944 7.02944 3 12 3Z"
                      >
                        <animate
                          fill="freeze"
                          attributeName="stroke-dashoffset"
                          dur="1.3s"
                          values="60;0"
                        />
                      </path>
                      <path
                        strokeDasharray="15"
                        strokeDashoffset="15"
                        d="M12 3C16.9706 3 21 7.02944 21 12"
                      >
                        <animate
                          fill="freeze"
                          attributeName="stroke-dashoffset"
                          dur="0.3s"
                          values="15;0"
                        />
                        <animateTransform
                          attributeName="transform"
                          dur="1.5s"
                          repeatCount="indefinite"
                          type="rotate"
                          values="0 12 12;360 12 12"
                        />
                      </path>
                    </g>
                  </svg>
                )}
              </Button>
            </div>
          </div>
        </form>


      )}


      {showOTP && (
        <>
          <div className="w-full md:w-[480px] mx-auto my-auto">
            <div
              style={{ textAlign: "center" }}
              className="mt-8 text-3xl font-bold text-default-900 mx-auto"
            >
              <h4 style={{ textAlign: "center" }} className="text-10 mb-4">
                {" "}
                {lang1("Please enter the OTP to verify your account")}
              </h4>
              <p
                style={{
                  color: "green",
                  textAlign: "center",
                  fontSize: "14px",
                }}
              >
                {lang1("A One-Time Password has been sent to")} {formatMobileNumber(mobile)}
              </p>
            </div>

            <form className="mt-8" onSubmit={onOTPVerify}>
              <div className="flex text-center">
                {otpFields.map((index) => (
                  <Input
                    key={`otp-code-${index}`}
                    type="text"
                    id={`otp${index}`}
                    name={`otp${index}`}
                    value={otp[index]}
                    onChange={(e) => handleChange(e, index)}
                    onKeyDown={(event) => handleKeyDown(index, event)}
                    maxLength={1}
                    className="w-10 h-10 sm:w-[60px] sm:h-16 rounded border-default-300 text-center text-2xl font-medium text-default-900"
                    ref={(ref) => (inputRefs.current[index] = ref)}
                  />
                ))}
              </div>

              <div
                style={{
                  display: "flex",
                  justifyContent: "flex-end",
                  marginTop: "1rem",
                }}
              > {lang1("Did not receive code?")} &nbsp;&nbsp;

                <a
                  href="#"
                  // onClick={otpResend}
                  style={{
                    color: isHovered ? "blue" : "#000",
                  }}
                  disabled={loadingOtpResend}
                >
                  <span
                    onClick={!isCounting ? otpResend : undefined}
                    className={`cursor-pointer rounded-lg ${isCounting ? 'text-gray-500' : 'text-blue-500'} ${isCounting ? '' : 'hover:text-blue-800'}`}
                  >
                    {isCounting ? `Resend in ${countdown}` : 'Resend OTP'}
                  </span>

                  {loadingOtpResend && (
                    <svg
                      xmlns="http://www.w3.org/2000/svg"
                      width="2em"
                      height="2em"
                      viewBox="0 0 24 24"
                    >
                      <g
                        fill="none"
                        stroke="currentColor"
                        strokeLinecap="round"
                        strokeWidth="2"
                      >
                        <path
                          strokeDasharray="60"
                          strokeDashoffset="60"
                          strokeOpacity="0.3"
                          d="M12 3C16.9706 3 21 7.02944 21 12C21 16.9706 16.9706 21 12 21C7.02944 21 3 16.9706 3 12C3 7.02944 7.02944 3 12 3Z"
                        >
                          <animate
                            fill="freeze"
                            attributeName="stroke-dashoffset"
                            dur="1.3s"
                            values="60;0"
                          />
                        </path>
                        <path
                          strokeDasharray="15"
                          strokeDashoffset="15"
                          d="M12 3C16.9706 3 21 7.02944 21 12"
                        >
                          <animate
                            fill="freeze"
                            attributeName="stroke-dashoffset"
                            dur="0.3s"
                            values="15;0"
                          />
                          <animateTransform
                            attributeName="transform"
                            dur="1.5s"
                            repeatCount="indefinite"
                            type="rotate"
                            values="0 12 12;360 12 12"
                          />
                        </path>
                      </g>
                    </svg>
                  )}
                </a>
              </div>

              <div className="mt-6">
                <Button
                  type="submit"
                  className="w-full"
                  size="lg"
                  disabled={loadingOtpVerify}
                >
                  {lang1("Verify Now")}
                  {loadingOtpVerify && (
                    <svg
                      xmlns="http://www.w3.org/2000/svg"
                      width="2em"
                      height="2em"
                      viewBox="0 0 24 24"
                    >
                      <g
                        fill="none"
                        stroke="currentColor"
                        strokeLinecap="round"
                        strokeWidth="2"
                      >
                        <path
                          strokeDasharray="60"
                          strokeDashoffset="60"
                          strokeOpacity="0.3"
                          d="M12 3C16.9706 3 21 7.02944 21 12C21 16.9706 16.9706 21 12 21C7.02944 21 3 16.9706 3 12C3 7.02944 7.02944 3 12 3Z"
                        >
                          <animate
                            fill="freeze"
                            attributeName="stroke-dashoffset"
                            dur="1.3s"
                            values="60;0"
                          />
                        </path>
                        <path
                          strokeDasharray="15"
                          strokeDashoffset="15"
                          d="M12 3C16.9706 3 21 7.02944 21 12"
                        >
                          <animate
                            fill="freeze"
                            attributeName="stroke-dashoffset"
                            dur="0.3s"
                            values="15;0"
                          />
                          <animateTransform
                            attributeName="transform"
                            dur="1.5s"
                            repeatCount="indefinite"
                            type="rotate"
                            values="0 12 12;360 12 12"
                          />
                        </path>
                      </g>
                    </svg>
                  )}
                </Button>
              </div>

            </form>
          </div>
        </>
      )}
    </>

  );
};

export default PasswordUpdate;
